@@ -21,7 +21,7 @@ from thonny.common import serialize_message, ToplevelCommand, \
     UserError
 from thonny.globals import get_workbench, get_runner
 import shlex
-from thonny import THONNY_USER_DIR
+from thonny import THONNY_USER_DIR, common
 from thonny.misc_utils import running_on_windows, running_on_mac_os, eqfn
 from shutil import which
 import shutil
@@ -57,7 +57,10 @@ class Runner:
         self._current_toplevel_command = None
         self._current_command = None
         
-        self._check_alloc_console()
+        try:
+            self._check_alloc_console()
+        except:
+            get_workbench().report_exception("Can't allocate console")
     
     def start(self):
         try:
@@ -351,7 +354,7 @@ class Runner:
                         break
                 except BackendTerminatedError as exc:
                     self._report_backend_crash(exc)
-                    self.reset_backend()
+                    self.kill_backend()
                     return
                 
                 if msg.get("SystemExit", False):
@@ -695,14 +698,24 @@ class CPythonProxy(BackendProxy):
         self._message_queue = None
     
     def _prepare_jedi(self):
-        """Make jedi available for the backend"""
+        """Make jedi (and parso) available for the backend"""
         
-        # Copy jedi
         import jedi
-        dirname = os.path.join(THONNY_USER_DIR, "jedi_" + str(jedi.__version__))
-        if not os.path.exists(dirname):
-            shutil.copytree(jedi.__path__[0], os.path.join(dirname, "jedi"))
-        return dirname
+        from thonny import jedi_utils 
+        target_dir = os.path.join(THONNY_USER_DIR, "jedi_" + str(jedi.__version__))
+        target_jedi_dir = os.path.join(target_dir, "jedi")
+        target_parso_dir = os.path.join(target_dir, "parso")
+        
+        if (not os.path.exists(target_jedi_dir)):
+            shutil.copytree(jedi.__path__[0], target_jedi_dir)
+        
+        # Newer jedis require also parso
+        if (not os.path.exists(target_parso_dir)
+            and jedi_utils.get_version_tuple() >= (0,11,0)):
+            import parso
+            shutil.copytree(parso.__path__[0], target_parso_dir)
+                
+        return target_dir
     
         # TODO: clean up old versions
     
@@ -712,23 +725,18 @@ class CPythonProxy(BackendProxy):
         self._message_queue = collections.deque()
     
         # prepare the environment
-        my_env = os.environ.copy()
         
         # Delete some environment variables if the backend is (based on) a different Python instance
-        if self._executable not in [
+        if self._executable in [
             this_python,
             this_python.replace("python.exe", "pythonw.exe"),
             this_python.replace("pythonw.exe", "python.exe"),
             get_private_venv_executable()]:
             
+            my_env = os.environ.copy()
+        else:
             # Keep only the variables, that are not related to Python
-            my_env = {name : my_env[name] for name in my_env 
-                      if "python" not in name.lower() and name not in ["TK_LIBRARY", "TCL_LIBRARY"]}
-            
-            # Remove variables used to tweak bundled Thonny-private Python
-            if using_bundled_python():
-                my_env = {name : my_env[name] for name in my_env
-                          if name not in ["SSL_CERT_FILE", "SSL_CERT_DIR", "LD_LIBRARY_PATH"]}
+            my_env = create_pythonless_environment()
         
         # variables controlling communication with the back-end process
         my_env["PYTHONIOENCODING"] = "ASCII" 
@@ -813,24 +821,48 @@ class CPythonProxy(BackendProxy):
     def _listen_stdout(self):
         #debug("... started listening to stdout")
         # will be called from separate thread
+        def publish_as_msg(data):
+            msg = parse_message(data)
+            if "cwd" in msg:
+                self.cwd = msg["cwd"]
+            self._message_queue.append(msg)
+            
+            if len(self._message_queue) > 100:
+                # Probably backend runs an infinite/long print loop.
+                # Throttle message thougput in order to keep GUI thread responsive.
+                sleep(0.1)
+        
         while True:
             data = self._proc.stdout.readline()
             #debug("... read some stdout data", repr(data))
             if data == '':
                 break
             else:
-                msg = parse_message(data)
-                if "cwd" in msg:
-                    self.cwd = msg["cwd"]
+                try:
+                    publish_as_msg(data)
+                except Exception:
+                    # Can mean the line was from subprocess, 
+                    # which can't be captured by stream faking.
+                    # NB! If subprocess printed it without linebreak,
+                    # then the suffix can be thonny message
                     
-                # TODO: it was "with self._state_lock:". Is it necessary?
-                self._message_queue.append(msg)
+                    parts = data.rsplit(common.MESSAGE_MARKER, maxsplit=1)
+                    
+                    # print first part as it is
+                    self._message_queue.append(dict(message_type="ProgramOutput",
+                                                data=parts[0],
+                                                stream_name="stdout"))
+                    
+                    if len(parts) == 2:
+                        second_part = common.MESSAGE_MARKER + parts[1]
+                        try:
+                            publish_as_msg(second_part)
+                        except Exception:
+                            # just print ...
+                            self._message_queue.append(dict(message_type="ProgramOutput",
+                                                        data=second_part,
+                                                        stream_name="stdout"))
                 
-                if len(self._message_queue) > 100:
-                    # Probably backend runs an infinite/long print loop.
-                    # Throttle message thougput in order to keep GUI thread responsive.
-                    sleep(0.1)
-
     def _listen_stderr(self):
         # stderr is used only for debugger debugging
         while True:
@@ -838,6 +870,11 @@ class CPythonProxy(BackendProxy):
             if data == '':
                 break
             else:
+                self._message_queue.append({
+                    "message_type" : "ProgramOutput",
+                    "stream_name" : "stderr",
+                    "data" : data,
+                })
                 debug("### BACKEND ###: %s", data.strip())
         
 
@@ -1093,6 +1130,20 @@ def _get_venv_info(venv_path):
     
     return result;
 
+def create_pythonless_environment():
+    my_env = os.environ.copy()
+    
+    # Keep only the variables, that are not related to Python
+    my_env = {name : my_env[name] for name in my_env 
+              if "PYTHON" not in name 
+              and name not in ["TK_LIBRARY", "TCL_LIBRARY"]}
+    
+    # Remove variables used to tweak bundled Thonny-private Python
+    if using_bundled_python():
+        my_env = {name : my_env[name] for name in my_env
+                  if name not in ["SSL_CERT_FILE", "SSL_CERT_DIR", "LD_LIBRARY_PATH"]}
+    
+    return my_env
 
 def using_bundled_python():
     return os.path.exists(os.path.join(
