@@ -30,7 +30,7 @@ AFTER_EXPRESSION_MARKER = "_thonny_hidden_after_expr"
 EXCEPTION_TRACEBACK_LIMIT = 100
 DEBUG = True    
 
-logger = logging.getLogger()
+logger = logging.getLogger("thonny.backend")
 info = logger.info
 
 class VM:
@@ -489,7 +489,7 @@ class VM:
         return result
            
     def _debug(self, *args):
-        print("VM:", *args, file=self._original_stderr)
+        logger.debug("VM: " + str(args))
     
     
     def _enter_io_function(self):
@@ -1041,6 +1041,17 @@ class FancyTracer(Executor):
                 node.tags.add("class=" + node.__class__.__name__)
             node.tags.add(tag)
         
+        # ignore module docstring if it is before from __future__ import
+        if (isinstance(root.body[0], ast.Expr)
+            and isinstance(root.body[0].value, ast.Str)
+            and len(root.body) > 1
+            and isinstance(root.body[1], ast.ImportFrom)
+            and root.body[1].module == "__future__"):
+            
+            add_tag(root.body[0], "ignore")
+            add_tag(root.body[0].value, "ignore")
+            add_tag(root.body[1], "ignore")
+        
         for node in ast.walk(root):
             
             # tag last children 
@@ -1078,7 +1089,12 @@ class FancyTracer(Executor):
             
             if isinstance(node, ast.Str):
                 add_tag(node, "StringLiteral")
-                
+            
+            if hasattr(ast, "JoinedStr") and isinstance(node, ast.JoinedStr):
+                # can't present children normally without
+                # ast giving correct locations for them
+                add_tag(node, "ignore_children")
+            
             if isinstance(node, ast.Num):
                 add_tag(node, "NumberLiteral")
             
@@ -1104,6 +1120,8 @@ class FancyTracer(Executor):
     
     def _should_instrument_as_expression(self, node):
         return (isinstance(node, _ast.expr)
+                and not getattr(node, "incorrect_range", False)
+                and "ignore" not in node.tags
                 and (not hasattr(node, "ctx") or isinstance(node.ctx, ast.Load))
                 # TODO: repeatedly evaluated subexpressions of comprehensions
                 # can be supported (but it requires some redesign both in backend and GUI)
@@ -1117,10 +1135,12 @@ class FancyTracer(Executor):
     
     def _should_instrument_as_statement(self, node):
         return (isinstance(node, _ast.stmt)
+                and not getattr(node, "incorrect_range", False)
+                and "ignore" not in node.tags
                 # Shouldn't insert anything before from __future__ import
                 # as this is not a normal statement
                 # https://bitbucket.org/plas/thonny/issues/183/thonny-throws-false-positive-syntaxerror
-                and (not isinstance(node, _ast.ImportFrom)
+                and (not isinstance(node, ast.ImportFrom)
                      or node.module != "__future__"))
     
     def _insert_statement_markers(self, root):
@@ -1143,7 +1163,7 @@ class FancyTracer(Executor):
                             self._insert_statement_markers(node)
                         new_list.append(node)
                         
-                        if isinstance(node, _ast.stmt):
+                        if self._should_instrument_as_statement(node):
                             # add after marker
                             new_list.append(self._create_statement_marker(node,
                                                                           AFTER_STATEMENT_MARKER))
@@ -1183,13 +1203,18 @@ class FancyTracer(Executor):
                         before_marker = tracer._create_simple_marker_call(node, BEFORE_EXPRESSION_MARKER)
                         ast.copy_location(before_marker, node)
                         
+                        if "ignore_children" in node.tags:
+                            transformed_node = node 
+                        else:
+                            transformed_node = ast.NodeTransformer.generic_visit(self, node)
+                        
                         # after marker
                         after_marker = ast.Call (
                             func=ast.Name(id=AFTER_EXPRESSION_MARKER, ctx=ast.Load()),
                             args=[
                                 before_marker,
                                 tracer._create_tags_literal(node),
-                                ast.NodeTransformer.generic_visit(self, node),
+                                transformed_node,
                                 tracer._create_location_literal(node.parent_node if hasattr(node, "parent_node") else None)
                             ],
                             keywords=[]
@@ -1245,7 +1270,7 @@ class FancyTracer(Executor):
         )
     
     def _debug(self, *args):
-        print("TRACER:", *args, file=self._vm._original_stderr)
+        logger.debug("TRACER: " + str(args))
 
 class CustomStackFrame:
     def __init__(self, frame, last_event, focus=None):
