@@ -544,6 +544,12 @@ class EnhancedText(TweakableText):
         self.edit_separator()        
         
     def _on_paste(self, e):
+        try:
+            if self.has_selection():
+                self.direct_delete("sel.first", "sel.last")
+        except:
+            pass
+        
         self._last_event_kind = "paste"
         self.edit_separator()    
         self.see("insert")  
@@ -829,4 +835,4 @@ if __name__ == "__main__":
     text.tag_configure('paren', underline=True)
     text.tag_add("paren", "4.6", "5.0")
     
-    root.mainloop()
\ No newline at end of file
+    root.mainloop()
